
const fs = require("fs")

/**
 * generate the html report containing the dynamic scritps we need to read the json report
 *
 * @param path: string
 * @param uncoveredLinesReport: string
 */
function generateHtmlReport(path,uncoveredLinesReport) {

    const uncoveredLinesHtml = createHtmlFromJsonReport(uncoveredLinesReport)

    const reportString = `
    <!DOCTYPE html>
    <html>
    <head>
        <title> Missing coverage report </title>
        <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
        <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/js/bootstrap.bundle.min.js" integrity="sha384-MrcW6ZMFYlzcLA8Nl+NtUVF0sA7MsXsP1UyJoMp4YLEuNSfAP+JcXn/tWtIaxVXM" crossorigin="anonymous"></script>
        <style type="text/css">
            .font-small {
                font-size: 10px;
            }
            .code {
                font-family: 'Courier New', monospace;
                font-size: 14px;
                margin-bottom: 5px;
            }
            .line-number {
                color: #A0A0A0;
            }
        </style>
    </head>
    <body>
        <div class="container">
            <h2>Uncovered Lines and Branches</h2>
            <p class="font-weight-light font-small">${new Date()}</p>
            <div id="uncovered" class="accordion">
                ${uncoveredLinesHtml}
            </div>
        </div>
    </body>
    </html>
    `
    fs.writeFileSync(path, reportString);
}

function createHtmlFromJsonReport(uncoveredLinesReport) {
    var uncoveredLinesHtml = ``
    var fileNumber = 1
    Object.keys(uncoveredLinesReport).forEach((filePath) => {
        var uncovLines = `<h5>Lines</h5><p class="font-weight-light font-small">Line Number | Line Content</P>`
        uncoveredLinesReport[filePath].uncoveredLines.forEach((line) => {
            uncovLines += `<p class="code"><span class="line-number">${pad(line.line, 4)}| </span>${line.content.substring(1, line.content.length)}</p>\n`
        })
        var uncovBranch = `<h5>Branches</h5><p class="font-weight-light font-small">Line Number | (Covered Branches / Total Branches) | Line Content</P>`
        uncoveredLinesReport[filePath].uncoveredBranches.forEach((branch) => {
            uncovBranch += `<p class="code"><span class="line-number">${pad(branch.line, 4)}| </span>${branch.missing} | ${branch.content.substring(1, branch.content.length)}</p>\n`
        })
        uncoveredLinesHtml += `<div class="accordion-item">
        <h2 class="accordion-header" id="heading${fileNumber}">
            <button class="accordion-button" type="button" data-bs-toggle="collapse" data-bs-target="#collapse${fileNumber}" aria-expanded="true" aria-controls="collapse${fileNumber}">
                ${filePath}
            </button>
        </h2>
            <div id="collapse${fileNumber}" class="accordion-collapse collapse ${fileNumber == 1 ? "show" : ""}" aria-labelledby="heading${fileNumber}" data-bs-parent="#uncovered">
                <div class="accordion-body">
                    ${uncovLines}
                    ${uncovBranch}
                </div>
            </div>
        </div>`
        fileNumber++
    })
    return uncoveredLinesHtml;
}
function pad(num, size) {
    num = num.toString();
    while (num.length < size) num = "0" + num;
    return num;
}
module.exports = {
    generateHtmlReport
};